package t9;
import java.util.Vector;

public class T9Tree {
	T9Node root; //root of the tree
	
	T9Tree() {
		root = new T9Node();
	}
	
	//pre: it will have the tree in any state - only guarantee is that there is the root node
	//	   so be aware that you will have to create nodes while traversing if they do not exist
	//post: changed the tree, such that it contains all the nodes necessary to encode the string
	//		saves the string at the node with the right encoding - see handout for more details
	public void addWord(String newword) {		
		T9Node cur_root = root;
		for (int i = 0; i < newword.length(); i++){		
			char current_letter = newword.charAt(i);
			int index = char2number(current_letter) - 2; //watch out for the -2 here			
			T9Node child = cur_root.children[index];
			if (child == null) {
				child = new T9Node();
				cur_root.children[index] = child; 				
			}
			cur_root = child;
		}
		//after this loop cur_root should point at the T9 Node that responds to the number sequence given by our string
		cur_root.addString(newword);		
	}
	
	
	//pre: a number 
	//post: the reversed of that number - eg. 12345 becomes 54321
	private long reverseNumber(long number){
		long n = number;
		long reverse = 0;
		while (n != 0){
			reverse = reverse * 10;
			reverse = reverse + n%10;
			n = n/10;
		}
		return reverse;
	}
	
	//pre: a word encoded as a number
	//post: returns all the strings contained in the tree that are encoded by this number (can be more then one!)
	public Vector<String> findWords(long number) {		
		long cur_number = reverseNumber(number);
		T9Node cur_root = root;
		while (cur_number >= 1 ) {
			int digit = (int) (cur_number % 10);			
			cur_number = cur_number / 10;
			
			cur_root = cur_root.children[digit - 2];
			if (cur_root == null)
				return null;								
		}
		return cur_root.words;
	}
	
	
	public void printString(Vector<String> strings) {
		if (strings != null)
		for (int i = 0; i < strings.size(); i++)
			System.out.println(strings.elementAt(i));
	}
	
	//this will turn a string into its t9 encoding
	public long string2number(String string){
		long number = 0;
		for(int i = 0; i < string.length(); i++){
			char cur_char = string.charAt(i);
			int num = char2number(cur_char);
			number = number * 10;
			number = number + num;
		}
		return number;
	}
	
	//This is the encoding we use (standard t9)
	public int char2number (char c) {
		int number = 2; //we do not handle error cases
		if (c == 'a' ||  c == 'b' || c == 'c')
			number = 2;
		if (c == 'd' ||  c == 'e' || c == 'f')
			number = 3;
		if (c == 'g' ||  c == 'h' || c == 'i')
			number = 4;
		if (c == 'j' ||  c == 'k' || c == 'l')
			number = 5;
		if (c == 'm' ||  c == 'n' || c == 'o')
			number = 6;
		if (c == 'p' ||  c == 'q' || c == 'r' || c == 's')
			number = 7;
		if (c == 't' ||  c == 'u' || c == 'v')
			number = 8;
		if (c == 'w' ||  c == 'x' || c == 'y' || c == 'z')
			number = 9;
		return number;
		
	}
	
	
}



